package net.knarcraft.blacksmith.config.blacksmith;

import net.knarcraft.blacksmith.config.GlobalSetting;
import net.knarcraft.blacksmith.config.SettingValueType;
import org.jetbrains.annotations.NotNull;

import java.util.Arrays;

/**
 * Settings which are the same for every blacksmith
 */
public enum GlobalBlacksmithSetting implements GlobalSetting {

    /**
     * The base price for repairing, regardless of durability
     *
     * <p>This allows specifying a price for each item, by setting basePrice.item_name.</p>
     */
    BASE_PRICE("basePrice.default", SettingValueType.POSITIVE_DOUBLE, 10.0, "basePrice"),

    /**
     * The base price for each durability point
     *
     * <p>If natural cost, this is the cost each missing durability point will add to the cost. If not natural cost,
     * this is the cost each present durability point will add to the cost. This allows specifying a price per
     * durability point value for each item, by setting pricePerDurabilityPoint.item_name</p>
     */
    PRICE_PER_DURABILITY_POINT("pricePerDurabilityPoint.default", SettingValueType.POSITIVE_DOUBLE,
            0.005, "pricePerDurabilityPoint"),

    /**
     * The price increase for each level of each present enchantment
     *
     * <p>This can be specified for each possible enchantment by setting enchantment-cost.enchantment_name</p>
     */
    ENCHANTMENT_COST("enchantmentCost.default", SettingValueType.POSITIVE_DOUBLE, 5.0,
            "enchantmentCost"),

    /**
     * Whether the cost should increase for damage taken, as opposed to increase for durability present
     */
    NATURAL_COST("useNaturalCost", SettingValueType.BOOLEAN, true, "useNaturalCost"),

    /**
     * Whether to show exact time when displaying the wait time for a reforging or the cool-down
     */
    SHOW_EXACT_TIME("showExactTime", SettingValueType.BOOLEAN, false, "showExactTime"),

    /**
     * The cost for repairing a chipped anvil
     */
    ANVIL_CHIPPED_COST("chippedAnvilReforgingCost", SettingValueType.POSITIVE_DOUBLE, 10.0,
            "chippedAnvilReforgingCost"),

    /**
     * The cost for repairing a damaged anvil
     */
    ANVIL_DAMAGED_COST("damagedAnvilReforgingCost", SettingValueType.POSITIVE_DOUBLE, 20.0,
            "damagedAnvilReforgingCost");

    private final String path;
    private final String parent;
    private final String commandName;
    private final Object value;
    private final SettingValueType valueType;

    /**
     * Instantiates a new setting
     *
     * @param path        <p>The full config path for this setting</p>
     * @param valueType   <p>The type of value used by this setting</p>
     * @param value       <p>The default value of this setting</p>
     * @param commandName <p>The name of the command used to change this setting</p>
     */
    GlobalBlacksmithSetting(String path, SettingValueType valueType, Object value, String commandName) {
        this.path = "blacksmith.global." + path;
        this.value = value;
        this.commandName = commandName;
        this.valueType = valueType;
        String[] pathParts = path.split("\\.");
        this.parent = String.join(".", Arrays.copyOfRange(pathParts, 0, pathParts.length - 1));
    }

    @Override
    public @NotNull String getPath() {
        return path;
    }

    @Override
    public @NotNull String getParent() {
        return parent;
    }

    @Override
    public @NotNull Object getDefaultValue() {
        return value;
    }

    @Override
    public @NotNull String getCommandName() {
        return commandName;
    }

    @Override
    public @NotNull SettingValueType getValueType() {
        return this.valueType;
    }

}
