package net.knarcraft.blacksmith.config.scrapper;

import net.citizensnpcs.api.util.DataKey;
import net.knarcraft.blacksmith.config.SettingValueType;
import net.knarcraft.blacksmith.config.SmithPreset;
import net.knarcraft.blacksmith.config.TraitSettings;
import net.knarcraft.blacksmith.util.ConfigHelper;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class ScrapperNPCSettings implements TraitSettings {
    private final Map<ScrapperNPCSetting, Object> currentValues = new HashMap<>();
    private final GlobalScrapperSettings globalScrapperSettings;

    /**
     * Instantiates a new scrapper NPC settings object
     *
     * @param globalScrapperSettings <p>The global settings object to get default values from</p>
     */
    public ScrapperNPCSettings(GlobalScrapperSettings globalScrapperSettings) {
        this.globalScrapperSettings = globalScrapperSettings;
    }

    /**
     * Loads variables from the given data key
     *
     * @param key <p>The data key to load variables from</p>
     */
    public void loadVariables(DataKey key) {
        for (ScrapperNPCSetting setting : ScrapperNPCSetting.values()) {
            if (key.keyExists(setting.getChildPath())) {
                currentValues.put(setting, key.getRaw(setting.getChildPath()));
            }
        }
    }

    /**
     * Saves variables to the given data key
     *
     * @param key <p>The data key to save variables to</p>
     */
    public void saveVariables(DataKey key) {
        for (ScrapperNPCSetting setting : ScrapperNPCSetting.values()) {
            key.setRaw(setting.getChildPath(), currentValues.get(setting));
        }
    }

    /**
     * Changes one setting to the given value
     *
     * @param setting  <p>The setting to change</p>
     * @param newValue <p>The new value of the setting</p>
     */
    public void changeSetting(ScrapperNPCSetting setting, Object newValue) {
        if (setting.getValueType() == SettingValueType.STRING_LIST ||
                setting.getValueType() == SettingValueType.REFORGE_ABLE_ITEMS) {
            //Workaround to make sure it's treated as the correct type
            currentValues.put(setting, newValue == null ? null : ConfigHelper.asStringList(newValue));
        } else {
            currentValues.put(setting, newValue);
        }
    }

    /**
     * Gets the raw current value of a setting
     *
     * @param setting <p>The setting to get the value of</p>
     * @return <p>The current value of the setting</p>
     */
    public Object getRawValue(ScrapperNPCSetting setting) {
        return currentValues.get(setting);
    }

    @Override
    public String getBusyWithPlayerMessage() {
        return asString(ScrapperNPCSetting.BUSY_WITH_PLAYER_MESSAGE);
    }

    @Override
    public String getBusyWorkingMessage() {
        return asString(ScrapperNPCSetting.BUSY_WITH_SALVAGE_MESSAGE);
    }

    @Override
    public String getStartWorkingMessage() {
        return asString(ScrapperNPCSetting.START_SALVAGE_MESSAGE);
    }

    /**
     * Gets the message to display when a blacksmith has successfully repaired an item
     *
     * @return <p>The reforge success message</p>
     */
    public String getSuccessMessage() {
        return asString(ScrapperNPCSetting.SUCCESS_SALVAGE_MESSAGE);
    }

    @Override
    public String getCoolDownUnexpiredMessage() {
        return asString(ScrapperNPCSetting.COOL_DOWN_UNEXPIRED_MESSAGE);
    }

    /**
     * The message displayed if a player presents a different item after seeing the price to salvage an item
     */
    public String getItemChangedMessage() {
        return asString(ScrapperNPCSetting.ITEM_UNEXPECTEDLY_CHANGED_MESSAGE);
    }

    /**
     * Gets the minimum delay used to wait for a salvaging to finish.
     *
     * @return <p>The minimum salvage delay</p>
     */
    public int getMinSalvageDelay() {
        return asInt(ScrapperNPCSetting.MIN_SALVAGE_DELAY);
    }

    /**
     * Gets the maximum delay used to wait for a salvaging to finish
     *
     * @return <p>The maximum salvage delay</p>
     */
    public int getMaxSalvageDelay() {
        return asInt(ScrapperNPCSetting.MAX_SALVAGE_DELAY);
    }

    /**
     * Gets the cool-down between each salvaging
     *
     * @return <p>The salvage cool-down</p>
     */
    public int getSalvageCoolDown() {
        return asInt(ScrapperNPCSetting.SALVAGE_COOL_DOWN);
    }

    /**
     * Gets whether an item should be dropped on the ground instead of being given to the player
     *
     * @return <p>Whether to drop reforged items on the ground</p>
     */
    public boolean getDropItem() {
        return asBoolean(ScrapperNPCSetting.DROP_ITEM);
    }

    @Override
    public boolean getDisableCoolDown() {
        return asInt(ScrapperNPCSetting.SALVAGE_COOL_DOWN) <= 0;
    }

    /**
     * Gets whether to disable the delay between starting reforging and the reforging finishing
     *
     * @return <p>Whether to disable the reforge delay</p>
     */
    public boolean getDisableDelay() {
        return asInt(ScrapperNPCSetting.MAX_SALVAGE_DELAY) <= 0;
    }

    /**
     * Gets the given value as an integer
     *
     * <p>This will throw an exception if used for a non-integer setting</p>
     *
     * @param setting <p>The setting to get the value of</p>
     * @return <p>The value of the given setting as an integer</p>
     */
    private int asInt(ScrapperNPCSetting setting) {
        return ConfigHelper.asInt(getValue(setting));
    }

    /**
     * Gets the string value of the given setting
     *
     * @param setting <p>The setting to get the value of</p>
     * @return <p>The value of the given setting as a string</p>
     */
    private String asString(ScrapperNPCSetting setting) {
        return getValue(setting).toString();
    }

    /**
     * Gets the boolean value of the given setting
     *
     * @param setting <p>The setting to get the value of</p>
     * @return <p>The value of the given setting as a boolean</p>
     */
    private boolean asBoolean(ScrapperNPCSetting setting) {
        return ConfigHelper.asBoolean(getValue(setting));
    }

    /**
     * Gets the value of a setting, using the default if not set
     *
     * @param setting <p>The setting to get the value of</p>
     * @return <p>The current value</p>
     */
    private Object getValue(ScrapperNPCSetting setting) {
        Object value = currentValues.get(setting);
        //If not set, use the default value from the config.yml file
        if (value == null) {
            Map<ScrapperNPCSetting, Object> defaultNPCSettings = globalScrapperSettings.getDefaultNPCSettings();
            if (defaultNPCSettings.containsKey(setting)) {
                value = defaultNPCSettings.get(setting);
            }
        }
        //If not set in config.yml, use the default value from the enum
        if (value == null) {
            value = setting.getDefaultValue();
        }
        return value;
    }

    /**
     * Replaces placeholders in the given reforge-able value
     *
     * @param stringList <p>The value specified by a user</p>
     * @return <p>The value with placeholders replaced</p>
     */
    private static List<String> replaceReforgeAblePresets(List<String> stringList) {
        List<String> newStrings = new ArrayList<>();
        for (String item : stringList) {
            if (item == null) {
                continue;
            }
            String replaced = SmithPreset.replacePreset(item);
            if (!replaced.equals(item)) {
                newStrings.addAll(List.of(replaced.split(",")));
            } else {
                newStrings.add(item);
            }
        }
        return newStrings;
    }

}
