package net.knarcraft.blacksmith.trait;

import net.citizensnpcs.api.npc.NPC;
import net.knarcraft.blacksmith.BlacksmithPlugin;
import net.knarcraft.blacksmith.config.scrapper.ScrapperNPCSettings;
import net.knarcraft.blacksmith.util.ItemHelper;
import net.knarcraft.blacksmith.util.SalvageHelper;
import org.bukkit.entity.LivingEntity;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.Objects;
import java.util.Random;

import static net.knarcraft.blacksmith.formatting.BlacksmithStringFormatter.sendNPCMessage;

/**
 * A representation of the session between a player and a blacksmith
 */
public class SalvageSession extends Session implements Runnable {

    private final ScrapperTrait scrapperTrait;
    private final NPC npc;
    private final ItemStack itemToSalvage;
    private final ScrapperNPCSettings config;
    private final List<ItemStack> salvage;
    private final int enchantmentLevels;

    /**
     * Instantiates a new session
     *
     * @param scrapperTrait <p>A reference to the blacksmith trait</p>
     * @param player        <p>The player initiating the session</p>
     * @param npc           <p>The Blacksmith NPC involved in the session</p>
     * @param config        <p>The config to use for the session</p>
     */
    SalvageSession(@NotNull ScrapperTrait scrapperTrait, @NotNull Player player, @NotNull NPC npc,
                   @NotNull ScrapperNPCSettings config) {
        super(player);
        this.scrapperTrait = scrapperTrait;
        this.npc = npc;
        this.itemToSalvage = player.getInventory().getItemInMainHand();
        this.config = config;
        // TODO: Implement the list of items to ignore when calculating salvage
        this.salvage = SalvageHelper.getSalvage(BlacksmithPlugin.getInstance().getServer(), this.itemToSalvage,
                new ArrayList<>());
        this.enchantmentLevels = SalvageHelper.getTotalEnchantmentLevels(this.itemToSalvage);
    }

    @Override
    public boolean isSessionInvalid() {
        // Prevent player from switching items during session
        ItemStack itemInHand = player.getInventory().getItemInMainHand();
        if (!itemToSalvage.equals(itemInHand)) {
            sendNPCMessage(this.npc, player, config.getItemChangedMessage());
            return true;
        }
        // TODO: Add economy check
        return false;
    }

    @Override
    protected int getActionDelay() {
        if (config.getMaxSalvageDelay() > 0) {
            //Finish the salvaging after a random delay between the max and min
            return new Random().nextInt(config.getMaxSalvageDelay()) + config.getMinSalvageDelay();
        } else {
            //Finish the salvaging as soon as possible
            return 0;
        }
    }

    /**
     * Runs the actual salvage which fixes the item and gives it back to the player
     */
    @Override
    public void run() {
        // TODO: Tell the player that the salvaging was successful, or cancel giveSalvage + give the original item back?

        // Give the player the result of the salvage
        giveSalvage();

        //Stop the reforged item from displaying in the blacksmith's hand
        if (npc.getEntity() instanceof Player) {
            ((Player) npc.getEntity()).getInventory().setItemInMainHand(null);
        } else {
            Objects.requireNonNull(((LivingEntity) npc.getEntity()).getEquipment()).setItemInMainHand(null);
        }

        //Mark this scrapper as available
        scrapperTrait.unsetSession();

        // Start cool-down
        Calendar wait = Calendar.getInstance();
        wait.add(Calendar.SECOND, config.getSalvageCoolDown());
        scrapperTrait.addCoolDown(player.getUniqueId(), wait);
    }

    /**
     * Gives the player the calculated salvage
     */
    private void giveSalvage() {
        // TODO: Find a better calculation than 1 enchantment level = 1 exp level
        // Gives the player back some of the XP used on an item
        player.giveExpLevels(enchantmentLevels);
        if (config.getDropItem() || !player.isOnline() || player.getInventory().firstEmpty() == -1) {
            // If the player isn't online, or the player cannot fit the item, drop the item to prevent it from
            // disappearing, even if drop item is disabled.
            for (ItemStack item : salvage) {
                player.getWorld().dropItemNaturally(npc.getEntity().getLocation(), item);
            }
        } else {
            for (ItemStack item : salvage) {
                if (ItemHelper.canFitItem(player.getInventory(), item)) {
                    player.getInventory().addItem(item);
                } else {
                    // If the player cannot fit all the salvage, drop it on the ground
                    player.getWorld().dropItemNaturally(player.getLocation(), item);
                }
            }
        }
    }

}
